!{\src2tex{textfont=tt}}
 !!****m* ABINIT/m_abi_linalg
!! NAME
!!  m_abi_linalg
!!
!! FUNCTION
!!  management of Linear Algebra wrappers routines
!! with support of different external library (scalapack,
!! elpa, plasma, magma, ... )
!!
!! COPYRIGHT
!!  Copyright (C) 2012-2013 ABINIT group (LNguyen,FDahm)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~ABINIT/Infos/copyright
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

module m_abi_linalg

  use defs_basis
  use defs_scalapack
  use m_errors
  use m_profiling
  use m_xmpi

#if defined HAVE_FC_ISO_C_BINDING
  use iso_c_binding, only: c_ptr
#endif


  implicit none
  !!***

#ifndef HAVE_FC_ISO_C_BINDING
  !This is needed for GPU linalg interface
  type, public :: c_ptr
     integer :: val
  end type c_ptr
#endif


  ! Working arrays for eigen problem
  integer :: eigen_s_maxsize
  integer :: eigen_d_maxsize
  integer :: eigen_c_maxsize
  integer :: eigen_z_maxsize

  integer :: eigen_s_lwork
  integer :: eigen_d_lwork
  integer :: eigen_c_lwork
  integer :: eigen_z_lwork

  real(sp),allocatable,dimension(:) :: eigen_c_rwork
  real(dp),allocatable,dimension(:) :: eigen_z_rwork
  real(sp),allocatable,dimension(:) :: eigen_s_work
  real(dp),allocatable,dimension(:) :: eigen_d_work
  complex(spc),allocatable,dimension(:) :: eigen_c_work
  complex(dpc),allocatable,dimension(:) :: eigen_z_work

#if defined(HAVE_LINALG_SCALAPACK)
  type(processor_scalapack),public :: abi_processor
  integer,public :: abi_communicator
#endif

  !----------------------------------------------------------------------
  !!***

  !Procedures ------------------------------------
  public :: abi_linalg_init                         ! Initialization and allocation routine
  public :: abi_linalg_finalize                  !  CleanuUp routine
  public :: abi_linalg_eigen_setmaxsize ! Allocate work array for eigen problem
  !----------------------------------------------------------------------

  !BLAS INTERFACE
  interface abi_xgemm
     module procedure abi_zgemm
     module procedure abi_zgemm_1d
     module procedure abi_zgemm_2d
     module procedure abi_dgemm
     module procedure abi_d2zgemm
  end interface
  !----------------------------------------------------------------------
  interface abi_xcopy
     module procedure abi_zcopy
     module procedure abi_zcopy_1d
     module procedure abi_dcopy
     module procedure abi_dcopy_1d
     module procedure abi_dcopy_2d
     module procedure abi_dcopy_0d_1d
     module procedure abi_dcopy_1d_0d
     module procedure abi_d2zcopy_2d
     module procedure abi_z2dcopy_2d
  end interface
  !----------------------------------------------------------------------
  interface abi_xtrsm
     module procedure abi_ztrsm
     module procedure abi_dtrsm
     module procedure abi_d2ztrsm
     module procedure abi_d2ztrsm_3d
  end interface
  !----------------------------------------------------------------------

  !LAPACK INTERFACE
  interface abi_xheev
     module procedure abi_dheev
     module procedure abi_cheev
     module procedure abi_zheev
     module procedure abi_dheev_alloc
     module procedure abi_cheev_alloc
     module procedure abi_zheev_alloc
  end interface
  !----------------------------------------------------------------------
  interface abi_xhegv
     module procedure abi_dhegv
     module procedure abi_chegv
     module procedure abi_zhegv
     module procedure abi_dhegv_alloc
     module procedure abi_chegv_alloc
     module procedure abi_zhegv_alloc
  end interface
  !----------------------------------------------------------------------
  interface abi_xhpev
     module procedure abi_dhpev
     module procedure abi_chpev
     module procedure abi_zhpev
     module procedure abi_dhpev_alloc_1d
     module procedure abi_dhpev_alloc_2d
     module procedure abi_chpev_alloc
     module procedure abi_zhpev_alloc
  end interface
  !----------------------------------------------------------------------
  interface abi_xhpgv
     module procedure abi_dhpgv
     module procedure abi_chpgv
     module procedure abi_zhpgv
     module procedure abi_dhpgv_alloc_1d
     module procedure abi_dhpgv_alloc_2d
     module procedure abi_chpgv_alloc
     module procedure abi_zhpgv_alloc
  end interface
  !----------------------------------------------------------------------
  interface abi_xpotrf
     module procedure abi_dpotrf
     module procedure abi_zpotrf
     module procedure abi_zpotrf_2d
     module procedure abi_d2zpotrf
  end interface
  !----------------------------------------------------------------------
  interface abi_xorthonormalize
     module procedure xorthonormalize
     module procedure orthonormalize
     module procedure zorthonormalize
  end interface
  !----------------------------------------------------------------------
#ifndef HAVE_GPU_CUDA
  !dummy routines replace gpu helper routines
  public :: alloc_on_gpu
  public :: copy_from_gpu
  public :: copy_on_gpu
  public :: dealloc_on_gpu
  public :: gpu_linalg_init
  public :: gpu_linalg_shutdown
  public :: gpu_xgemm
  public :: gpu_xtrsm
#endif
  public :: gpu_xorthonormalize
  !----------------------------------------------------------------------

CONTAINS  !===========================================================
!!***


!!****f* m_abi_linalg/abi_linalg_init
!! NAME
!! abi_linalg_init
!!
!! FUNCTION
!! Initalization of linear algebra environnement
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2013 ABINIT group (LNguyen,FDahm)
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!
!! PARENTS
!!      compute_kgb_indicator,driver
!!
!! CHILDREN
!!      end_scalapack,gpu_linalg_shutdown,plasma_finalize,xcomm_free
!!
!! SOURCE
!!
 subroutine abi_linalg_init(comm_scalapack,eigen_group_size,max_eigen_pb_size,my_rank,&
&                           only_scalapack) ! optional parameter

#if defined HAVE_LINALG_PLASMA
 use m_xomp
 use plasma
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_linalg_init'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 integer,intent(in) :: comm_scalapack,eigen_group_size,max_eigen_pb_size,my_rank
 logical,intent(in),optional :: only_scalapack

!Local variables ------------------------------
#if defined HAVE_LINALG_SCALAPACK
 integer :: abi_info1,eigen_group_color,rank
#endif
#if defined HAVE_LINALG_PLASMA
 integer :: abi_info2,core_id,num_cores,num_cores_node
 integer,allocatable :: affinity(:)
#endif

!******************************************************************

#if defined HAVE_LINALG_SCALAPACK
!Scalapack initalization
 if (eigen_group_size>0) then
   abi_communicator = comm_scalapack
   rank=xcomm_rank(comm_scalapack)
   eigen_group_color=rank/eigen_group_size
   call xmpi_comm_split(comm_scalapack,eigen_group_color,rank,abi_communicator,abi_info1)
   call init_scalapack(abi_processor,abi_communicator)
 else
  abi_communicator=xmpi_comm_null
 end if
#endif

 if (present(only_scalapack)) then
   if (only_scalapack) return
 end if

#if defined HAVE_LINALG_PLASMA
!Plasma Initialization
!Because use of hybrid use of mpi+openmp+plasma,
!  we need to set manually the thread bindings policy
!  to avoid conflicts between mpi process due to plasma
 num_cores=xomp_get_max_threads()
 num_cores_node=xomp_get_num_cores_node()
 ABI_ALLOCATE(affinity,(num_cores))
 do core_id =1,num_cores
   affinity(core_id) = MOD(my_rank*num_cores + (core_id-1), num_cores_node)
 end do
 call PLASMA_Init_Affinity(num_cores,affinity(1),abi_info2)
 ABI_DEALLOCATE(affinity)
#endif

#if defined HAVE_GPU_CUDA
!Cublas initialization
 call gpu_linalg_init
#endif

!Allocation of work sizes
 eigen_s_maxsize = 0
 eigen_d_maxsize = 0
 eigen_c_maxsize = 0
 eigen_z_maxsize = 0
 eigen_s_lwork   = 0
 eigen_d_lwork   = 0
 eigen_c_lwork   = 0
 eigen_z_lwork   = 0
 call abi_linalg_eigen_setmaxsize(max_eigen_pb_size)

 return

 ABI_UNUSED(comm_scalapack)
 ABI_UNUSED(eigen_group_size)
 ABI_UNUSED(my_rank)

 end subroutine abi_linalg_init
!!***

!!****f* m_abi_linalg/abi_linalg_eigen_setmaxsize
!! NAME
!! abi_linalg_eigen_setmaxsize
!!
!! FUNCTION
!!
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2013 ABINIT group (LNguyen,FDahm)
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!
!! PARENTS
!!      m_abi_linalg
!!
!! CHILDREN
!!      end_scalapack,gpu_linalg_shutdown,plasma_finalize,xcomm_free
!!
!! SOURCE
!!
 subroutine abi_linalg_eigen_setmaxsize(max_eigen_pb_size)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_linalg_eigen_setmaxsize'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 integer,intent(in) :: max_eigen_pb_size

!Local variables ------------------------------

!******************************************************************

!Single precision work arrays (real and complex)
 eigen_s_maxsize = max_eigen_pb_size
 eigen_s_lwork = max(1,3*eigen_s_maxsize)
 if(allocated(eigen_s_work)) then
   ABI_DEALLOCATE(eigen_s_work)
 end if
 ABI_ALLOCATE(eigen_s_work,(max(1,eigen_s_lwork)))

 eigen_c_maxsize = max_eigen_pb_size
 eigen_c_lwork = max(1,2*eigen_c_maxsize-1)
 if(allocated(eigen_c_work)) then
   ABI_DEALLOCATE(eigen_c_work)
 end if

 ABI_ALLOCATE(eigen_c_work,(max(1,eigen_c_lwork)))
 if(allocated(eigen_c_rwork)) then
   ABI_DEALLOCATE(eigen_c_rwork)
 end if
 ABI_ALLOCATE(eigen_c_rwork,(max(1, 3*eigen_c_maxsize-2)))

!Double precision work arrays (real and complex)
 eigen_d_maxsize = max_eigen_pb_size
 eigen_d_lwork = max(eigen_d_lwork,2*(2*eigen_d_maxsize - 1)) !for zh[ep][eg]v
 eigen_d_lwork = max(eigen_d_lwork,(3*eigen_d_maxsize))       !for ds[ep][eg]v
#if defined HAVE_LINALG_MAGMA
 eigen_d_lwork = max(eigen_d_lwork,2*(2*(eigen_d_maxsize**2) + 6*eigen_d_maxsize +1))!for magma_zhe[eg]v
 eigen_d_lwork = max(eigen_d_lwork, eigen_d_maxsize**2 + 33*eigen_d_maxsize)         !for magma_dsy[eg]v
#endif
 if(allocated(eigen_d_work)) then
   ABI_DEALLOCATE(eigen_d_work)
 end if
 ABI_ALLOCATE(eigen_d_work,(max(1,eigen_d_lwork)))

 eigen_z_maxsize = max_eigen_pb_size
 eigen_z_lwork = max(1,2*eigen_z_maxsize-1)
 if(allocated(eigen_z_work)) then
   ABI_DEALLOCATE(eigen_z_work)
 end if
 ABI_ALLOCATE(eigen_z_work,(max(1,eigen_z_lwork)))
 if(allocated(eigen_z_rwork)) then
   ABI_DEALLOCATE(eigen_z_rwork)
 end if

#if defined HAVE_LINALG_MAGMA
 ABI_ALLOCATE(eigen_z_rwork,(max(1, 2*(eigen_z_maxsize**2) + 5*eigen_z_maxsize + 1)))
#else
 ABI_ALLOCATE(eigen_z_rwork,(max(1, 3*eigen_z_maxsize-2)))
#endif

 end subroutine abi_linalg_eigen_setmaxsize
!!***

!!****f* m_abi_linalg/abi_linalg_finalize
!! NAME
!! abi_linalg_finalize
!!
!! FUNCTION
!!
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2013 ABINIT group (LNguyen,FDahm)
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!
!! PARENTS
!!      compute_kgb_indicator,driver
!!
!! CHILDREN
!!      end_scalapack,gpu_linalg_shutdown,plasma_finalize,xcomm_free
!!
!! SOURCE
!!
 subroutine abi_linalg_finalize(only_scalapack) ! optional argument

#if defined HAVE_LINALG_PLASMA
 use plasma
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_linalg_finalize'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 logical,intent(in),optional :: only_scalapack

!Local variables ------------------------------
#if defined HAVE_LINALG_PLASMA
    integer :: abi_info
#endif

!******************************************************************

#if defined HAVE_LINALG_SCALAPACK
 if (abi_communicator/=xmpi_comm_null) then
   call xcomm_free(abi_communicator)
   call end_scalapack(abi_processor)
 end if
#endif
 if (present(only_scalapack)) then
   if (only_scalapack) return
 end if

#if defined HAVE_LINALG_PLASMA
 call PLASMA_Finalize(abi_info)
#endif

#if defined HAVE_GPU_CUDA
 call gpu_linalg_shutdown
#endif

!Memory freeing
 if(allocated(eigen_s_work)) then
   ABI_DEALLOCATE(eigen_s_work)
 end if
 if(allocated(eigen_d_work)) then
   ABI_DEALLOCATE(eigen_d_work)
 end if
 if(allocated(eigen_c_work)) then
   ABI_DEALLOCATE(eigen_c_work)
 end if
 if(allocated(eigen_c_rwork)) then
   ABI_DEALLOCATE(eigen_c_rwork)
 end if
 if(allocated(eigen_z_work)) then
   ABI_DEALLOCATE(eigen_z_work)
 end if
 if(allocated(eigen_z_rwork)) then
   ABI_DEALLOCATE(eigen_z_rwork)
 end if

 eigen_s_maxsize = 0
 eigen_d_maxsize = 0
 eigen_c_maxsize = 0
 eigen_z_maxsize = 0
 eigen_s_lwork   = 0
 eigen_d_lwork   = 0
 eigen_c_lwork   = 0
 eigen_z_lwork   = 0

 end subroutine abi_linalg_finalize
!!***


!----------------------------------------------------------------------

! Include files providing wrappers for some of the most commonly used BLAS & LAPACK routines

! *********************************************************************!
! ******************* BLAS_XGEMM interface ****************************!
! *********************************************************************!
#include "abi_xgemm.f90"

! *********************************************************************!
! ******************** BLAS_XCOPY interface ***************************!
! *********************************************************************!
#include "abi_xcopy.f90"

! *********************************************************************!
! ******************** BLAS_XTRSM interface ***************************!
! *********************************************************************!
#include "abi_xtrsm.f90"

! *********************************************************************!
! ********************* LAPACK XHEEV  interface ***********************!
! *********************************************************************!
#include "abi_xheev.f90"

! *********************************************************************!
! ********************* LAPACK XHEGV  interface ***********************!
! *********************************************************************!
#include "abi_xhegv.f90"

! *********************************************************************!
! ********************* LAPACK XHPEV  interface ***********************!
! *********************************************************************!
#include "abi_xhpev.f90"

! *********************************************************************!
! ********************* LAPACK XHPGV  interface ***********************!
! *********************************************************************!
#include "abi_xhpgv.f90"

! *********************************************************************!
! ****************** LAPACK XPOTRF  interface *************************!
! *********************************************************************!
#include "abi_xpotrf.f90"

! *********************************************************************!
! ************  ABINIT Orthonormalization interface *******************!
! *********************************************************************!
#include "abi_xorthonormalize.f90"

! *********************************************************************!
! ******************    GPU LINALG interface  *************************!
! *********************************************************************!
#include "abi_gpu_linalg.f90"

!----------------------------------------------------------------------

end module m_abi_linalg
!!***
