!{\src2tex{textfont=tt}}
!!****f* ABINIT/finddistrproc
!! NAME
!! finddistrproc
!!
!! FUNCTION
!!   Given a total number of processors, find a suitable distribution
!!   that fill all the different levels of parallelization
!!   (npimage, npkpt, npspinor, npband, npfft, bandpp)
!!   Also determine parameters of parallel Linear Algebra routines
!!   (use_slk, np_slk, gpu_linalg_limit)
!!
!! COPYRIGHT
!! Copyright (C) 1999-2013 ABINIT group (FJ,MT,FD)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt.
!!
!! INPUTS
!!  dtset = information about input parameter
!!  mpi_enreg=informations about MPI parallelization
!!  mband=maximum number of bands.
!!  tread(10)=flags indicating wether parallel input parameters were read from input file
!!            tread(1): paral_kgb      tread(6): npband
!!            tread(2): npimage        tread(7): bandd
!!            tread(3): npkpt          tread(8): use_slk
!!            tread(4): nspinor        tread(9): np_slk
!!            tread(5): npfft          tread(10): gpu_linalg_limit
!!
!! SIDE EFFECTS
!!  iexit= if incremented, an exit is required
!!
!! SIDE EFFECTS
!!  dtset%paral_kgb= flag for band-fft parallelism
!!  dtset%npimage  = number of processors for parallelisation over image
!!  dtset%npspinor = number of processors for parallelisation on k points
!!  dtset%npkpt    = number of processors for parallelisation on k points
!!  dtset%npfft    = number of processors for parallelisation on fft grid
!!  dtset%npband   = number of processors for parallelisation on bands
!!  dtset%bandpp   = internal parameter for lobpcg parallelisation algorithm
!!  dtset%use_slk  = flag for ScalaPAck use
!!  dtset%np_slk   = number of processors used in ScaLapack routines
!!  dtset%gpu_linalg_limit=threshold activating Linear Algebra on GPU
!!
!! PARENTS
!!      mpi_setup
!!
!! CHILDREN
!!      compute_kgb_indicator,initmpi_world,kpgcount,metric,mkrdim,sort_dp,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

 subroutine finddistrproc(dtset,iexit,mband,mpi_enreg,tread)

 use defs_basis
 use defs_abitypes
 use m_profiling
 use m_errors
 use m_xmpi
 use m_kpgsph, only : kpgcount

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'finddistrproc'
 use interfaces_14_hidewrite
 use interfaces_28_numeric_noabirule
 use interfaces_41_geometry
 use interfaces_51_manage_mpi
 use interfaces_57_iovars, except_this_one => finddistrproc
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: mband
 integer,intent(inout) :: iexit
 type(dataset_type),intent(inout) :: dtset
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays
 integer,intent(in) :: tread(10)

!Local variables-------------------------------
!scalars
!128 should be a reasonable maximum for npfft (scaling is very poor for npfft>20)
 integer,parameter :: NPFMAX=128
 integer,parameter :: MAXCOUNT=250,MAXBENCH=25,NPF_CUTOFF=20
 integer :: bpp,icount,ii,imin,jj,mcount,mcount_eff,mm,mpw
 integer :: n2,n3,ncount,nimage_eff,nkpt_eff,nn
 integer :: npi_min,npi_max,npk_min,npk_max,nps_min,nps_max
 integer :: npf_min,npf_max,npb_min,npb_max,bpp_min,bpp_max
 integer :: npi,npk,nps,npf,npb,nproc,nproc1,nprocmin
 integer :: np_slk,use_linalg_gpu
 logical,parameter :: new_version=.true.
 logical :: first_bpp
 real(dp):: acc_i,acc_k,acc_kgb,acc_kgb_0,acc_s,cutoff,ecut_eff,speedup,ucvol,weight0
 character(len=500) :: message
!arrays
 integer :: idum(1),ngmax(3),ngmin(3)
 integer,allocatable :: isort(:),my_distp(:,:)
 real(dp) :: gmet(3,3),gprimd(3,3),rmet(3,3),rprimd(3,3)
 real(dp),allocatable :: weight(:)

!no abirules
!Expected linear speedup for a nn-sized problem and mm processes
 speedup(nn,mm)=(one*nn)/(one*((nn/mm)+merge(0,1,mod(nn,mm)==0)))
!Cut-off function for npfft
 cutoff(nn)= &
&    0.2_dp+(one-0.2_dp)*(sin((pi*(nn-NPF_CUTOFF))/(one*(NPFMAX-NPF_CUTOFF))) &
&                           /((pi*(nn-NPF_CUTOFF))/(one*(NPFMAX-NPF_CUTOFF))))**2

!******************************************************************

 DBG_ENTER("COLL")

 nproc=mpi_enreg%nproc
 if (xmpi_paral==1.and.dtset%paral_kgb <0) nproc=-dtset%paral_kgb
 if (xmpi_paral==0.and.dtset%paral_kgb>=0) nproc=1

 if (dtset%paral_kgb>=0) then
   if (nproc==1) then
     if (tread(1)==0.or.xmpi_paral==0) dtset%paral_kgb= 0
     if (tread(2)==0.or.xmpi_paral==0) dtset%npimage  = 1
     if (tread(3)==0.or.xmpi_paral==0) dtset%npspinor = 1
     if (tread(4)==0.or.xmpi_paral==0) dtset%npkpt    = 1
     if (tread(5)==0.or.xmpi_paral==0) dtset%npfft    = 1
     if (tread(6)==0.or.xmpi_paral==0) dtset%npband   = 1
     if (tread(7)==0.or.xmpi_paral==0) dtset%bandpp   = 1
     if (tread(8)==0.or.xmpi_paral==0) dtset%use_slk  = 0
     if (tread(9)==0.or.xmpi_paral==0) dtset%np_slk   = 1000000
     return
   end if
   if (dtset%optdriver/=RUNL_GSTATE.or.dtset%iscf<0.or.dtset%usewvl==1) then
     dtset%paral_kgb= 0
     dtset%npimage  = max(1,dtset%npimage)
     dtset%npspinor = max(1,dtset%npspinor)
     dtset%npkpt    = max(1,dtset%npkpt)
     dtset%npfft    = max(1,dtset%npfft)
     dtset%npband   = max(1,dtset%npband)
     dtset%bandpp   = max(1,dtset%bandpp)
     return
   end if
 end if

 nprocmin=2
 if (xmpi_paral==1.and.dtset%paral_kgb>=0) nprocmin=max(2,nproc-100)

!Need the metric tensor
 call mkrdim(dtset%acell_orig(1:3,1),dtset%rprim_orig(1:3,1:3,1),rprimd)
 call metric(gmet,gprimd,-1,rmet,rprimd,ucvol)

!Determine some quantities related to plane waves
!  - Crude estimation of the number of PW
!  - Number of G vectors in each direction
 ecut_eff = dtset%ecut*dtset%dilatmx**2
 mpw = nint(ucvol*((two*ecut_eff)**1.5_dp)/(six*pi**2)) ! Crude estimation
 if (all(dtset%istwfk(1:dtset%nkpt)>1)) mpw=mpw/2+1
 call kpgcount(ecut_eff,dtset%exchn2n3d,gmet,dtset%istwfk, &
&              dtset%kpt,ngmax,ngmin,dtset%nkpt)
 write(message,'(a,i8)') ' getmpw sequential formula gave: ',mpw
 call wrtout(std_out,message,'COLL')

 write(message, '(2a,i6)' )  ch10,&
& ' Computing all possible proc distrib for this input with nproc less than ',nproc
 call wrtout(std_out,message,'COLL')

!Parallelization over images
 nimage_eff=dtset%ndynimage
 if (dtset%ntimimage<=1) nimage_eff=dtset%nimage
 npi_min=max(1,dtset%npimage)
 npi_max=min(nproc,nimage_eff)
 if (tread(2)==1) npi_max=dtset%npimage

!Parallelization over k-points and spin components
 nkpt_eff=dtset%nkpt*dtset%nsppol
 npk_min=max(1,dtset%npkpt)
 npk_max=min(nproc,nkpt_eff)
 if (tread(3)==1) npk_max=dtset%npkpt

!Parallelization over spinorial components
 nps_min=max(1,dtset%npspinor)
 nps_max=min(nproc,dtset%nspinor)
 if (tread(4)==1) nps_max=dtset%npspinor

!KGB Parallelization
!>> FFT level
 n2=dtset%ngfft(2)
 n3=dtset%ngfft(2)
 npf_min=max(1,dtset%npfft)
 npf_min=min(npf_min,ngmin(2))
 npf_max=min(nproc,NPFMAX)
 if (tread(5)==1) then
   npf_max=dtset%npfft
   if (npf_max>ngmin(2)) then
     write(message,'(a,i8)') &
&        "Value of npfft given in input file is too for the FFT grid!",ch10,&
&        "Action: decrease npfft or increase FFT grid (ecut, ngfft, ...)."
     MSG_ERROR(message)
   end if 
 end if
 npf_max=min(npf_max,ngmin(2))
 if (dtset%use_gpu_cuda==1) then
   npf_min=1;npf_max=1
 end if
!>> Band level
 npb_min=max(1,dtset%npband)
 npb_max=min(nproc,mband)
 if (tread(6)==1) npb_max=dtset%npband
!>> banddp level
 bpp_min=max(1,dtset%bandpp)
 bpp_max=max(4,nint(mband/10.)) ! reasonnable bandpp max
 if (tread(7)==1) bpp_max=dtset%bandpp

!Disable KGB parallelisation in some cases : either in input file or for DMFT
 if ((dtset%paral_kgb==0.and.tread(1)==1) .or. &
& ((dtset%usedmft == 1).and.(dtset%nstep>1)))  then
   nps_min=1; nps_max=1
   npf_min=1; npf_max=1
   npb_min=1; npb_max=1
   bpp_min=1; bpp_max=1
 end if

!Print title
 if(mpi_enreg%me==0) then
   write(message, '(8(a12,a1),a,8(i4,a4,i4,a1))' )  &
   'npimage','|','npkpt','|','npspinor','|','npfft','|','npband','|',' bandpp ' ,'|','nproc','|','weight','|', ch10, &
   npi_min,' -> ',npi_max,'|',      npk_min,' -> ',npk_max,'|',      nps_min,' -> ',nps_max,'|', &
   npf_min,' -> ',npf_max,'|',      npb_min,' -> ',npb_max,'|',      bpp_min,' -> ',bpp_max,'|', &
   nprocmin,' -> ',nproc,'|', 1 ,' -> ',nproc,'|'
   call wrtout(std_out,message,'COLL')
   if(dtset%paral_kgb < 0) then
     call wrtout(ab_out,message,'COLL')
   end if
 end if

!Allocate lists
 ABI_ALLOCATE(my_distp,(10,MAXCOUNT))
 ABI_ALLOCATE(weight,(MAXCOUNT))
 my_distp(1:7,:)=0;weight(:)=zero
 my_distp(8,:)=dtset%use_slk
 my_distp(9,:)=dtset%np_slk
 my_distp(10,:)=dtset%gpu_linalg_limit
 icount=0;imin=1

!Loop over all possibilities
!Computation of weight~"estimated acceleration"
 if (new_version) then

!  ======= NEW VERSION ========
   do npi=npi_min,npi_max
     acc_i=one;if (npi>1) acc_i=0.99_dp*speedup(nimage_eff,npi)

     do npk=npk_min,npk_max
       acc_k=one;if (npk>1) acc_k=0.96_dp*speedup(nkpt_eff,npk)

       do nps=nps_min,nps_max
         acc_s=one;if (nps>1) acc_s=0.85_dp*speedup(dtset%nspinor,nps)

         do npf=npf_min,npf_max
!          -> npf should divide ngfft if set (if unset, ngfft=0 so the modulo test is ok)
           if((modulo(n2,npf)>0).or.(modulo(n3,npf)>0)) cycle
!          -> npf should be only divisible by 2, 3 or 5
           ii=npf
           do while (modulo(ii,2)==0)
             ii=ii/2
           end do
           do while (modulo(ii,3)==0)
             ii=ii/3
           end do
           do while (modulo(ii,5)==0)
             ii=ii/5
           end do
           if(ii/=1) cycle

           do npb=npb_min,npb_max
             nproc1=npi*npk*nps*npf*npb
             if (nproc1<nprocmin)     cycle
             if (nproc1>nproc)        cycle
             if (modulo(mband,npb)>0) cycle

!            Base speedup
             acc_kgb_0=one;if (npb*npf>1) acc_kgb_0=0.7_dp*speedup(mpw,(npb*npf))

             if (npb*npf>4) then
!              Promote npb=npf
               acc_kgb_0=acc_kgb_0*min((one*npf)/(one*npb),(one*npb)/(one*npf))
!              Promote npf<=20
               if (npf>20) acc_kgb_0=acc_kgb_0*cutoff(npf)
             end if

             first_bpp=.true.
             do bpp=bpp_min,bpp_max
               if (modulo(mband/npb,bpp)>0) cycle
               if ((bpp>1).and.(modulo(bpp,2)>0)) cycle
               if (one*npb*bpp >max(1.,mband/3.)) cycle
               if (npb*npf<=4.and.(.not.first_bpp)) cycle
               first_bpp=.false.

               acc_kgb=acc_kgb_0
!              Promote bpp*npb>mband/3
               if (npb*npf>4) acc_kgb=acc_kgb*(one-(three*bpp*npb)/(one*mband))

!              Resulting speedup
!              weight0=acc_i*acc_k*acc_s*acc_kgb
               weight0=nproc1*(acc_i+acc_k+acc_s+acc_kgb)/(npi+npk+nps+(npf*npb))

!              Store data
               icount=icount+1
               if (icount<=MAXCOUNT) then
                 my_distp(1:7,icount)=(/npi,npk,nps,npf,npb,bpp,nproc1/)
                 weight(icount)=weight0
                 if (weight0<weight(imin)) imin=icount
               else
                 if (weight0>weight(imin)) then
                   my_distp(1:7,imin)=(/npi,npk,nps,npf,npb,bpp,nproc1/)
                   weight(imin)=weight0
                   idum=minloc(weight);imin=idum(1)
                 end if
               end if

             end do ! bpp
           end do ! npb
         end do ! npf
       end do ! nps
     end do ! npk
   end do ! npi
 else

!  ======= OLD VERSION ========
   do npi=npi_min,npi_max
     acc_i=one
     if (npi>1) acc_i = 0.99_dp*nimage_eff/((nimage_eff+npi-1)/npi)

     do npk=npk_min,npk_max
       acc_k=one
       if (npk>1) acc_k = 0.96_dp*nkpt_eff/((nkpt_eff+npk-1)/npk)

       do nps=nps_min,nps_max
         acc_s=one
         if (nps>1) acc_s = 0.85_dp*dtset%nspinor/ ((dtset%nspinor+nps-1)/nps)

         do npf=npf_min,npf_max
!          -> npf should divide ngfft if set (if unset, ngfft=0 so the modulo test is ok)
           if((modulo(n2,npf)>0).or.(modulo(n3,npf)>0)) cycle
!          -> npf should be only divisible by 2, 3 or 5
           npb=npf ! Note that here, npb is used as a temp var
           do while (modulo(npb,2)==0)
             npb=npb/2
           end do
           do while (modulo(npb,3)==0)
             npb=npb/3
           end do
           do while (modulo(npb,5)==0)
             npb=npb/5
           end do
           if(npb/=1) cycle

           do npb=npb_min,npb_max
             nproc1=npi*npk*nps*npf*npb
             if (nproc1<nprocmin) cycle
             if (nproc1>nproc) cycle
             if(modulo(mband,npb)>0) cycle

             do bpp=bpp_max,bpp_min,-1
               if(modulo(mband/npb,bpp)>0) cycle
               if((bpp>1).and.(modulo(bpp,2)>0)) cycle
               if (1.*npb*bpp >max(1.,mband/3.)) cycle

               acc_kgb=one
               if (npb*npf>4) then
                 acc_kgb=min((one*npf)/(one*npb),(one*npb)/(one*npf))  * &
                 (mpw/(mpw/(npb*npf)))*(one-(three*bpp*npb)/mband)
               else if (npb*npf >1) then
                 acc_kgb=(mpw*mband/(mband*mpw/(npb*npf)))*0.7_dp
               end if

!              STILL TO IMPROVE: BANDPP=1 is always the best....

!              Weight average for efficiency and estimated acceleration
               weight0=(acc_i+acc_k+acc_s+acc_kgb)/(npi+npk+nps+(npf*npb))
               weight0=weight0*nproc1

!              Store data
               icount=icount+1
               if (icount<=MAXCOUNT) then
                 my_distp(1:7,icount)=(/npi,npk,nps,npf,npb,bpp,nproc1/)
                 weight(icount)=weight0
                 if (weight0<weight(imin)) imin=icount
               else
                 if (weight0>weight(imin)) then
                   my_distp(1:7,imin)=(/npi,npk,nps,npf,npb,bpp,nproc1/)
                   weight(imin)=weight0
                   idum=minloc(weight);imin=idum(1)
                 end if
               end if

             end do ! bpp
           end do ! npb
         end do ! npf
       end do ! nps
     end do ! npk
   end do ! npi

!  New or old version
 end if

 mcount_eff=icount
 mcount=min(mcount_eff,MAXCOUNT)

 if (mcount==0) then
   write(message,'(a,i5,2a,i6,a)')  &
   'Your input dataset does not let Abinit find an appropriate process distribution with nproc=',nproc,ch10, &
   ' Try to comment all the np* vars and set paral_kgb=',-nproc,' to have advices on process distribution.'
   MSG_WARNING(message)
!  Override here the 0 default value changed in indefo1
   dtset%npimage  = max(1,dtset%npimage)
   dtset%npkpt    = max(1,dtset%npkpt)
   dtset%npspinor = max(1,dtset%npspinor)
   dtset%npfft    = max(1,dtset%npfft)
   dtset%npband   = max(1,dtset%npband)
   dtset%bandpp   = max(1,dtset%bandpp)
   ABI_DEALLOCATE(my_distp)
   ABI_DEALLOCATE(weight)
   return
 end if

!Sort data by increasing weight
 ABI_ALLOCATE(isort,(mcount))
 isort=(/(ii,ii=1,mcount)/)
 call sort_dp(mcount,weight,isort,tol6)

 ncount=mcount;if (dtset%paral_kgb>=0) ncount=min(mcount,5)
 if (mpi_enreg%me==0) then
   do jj=mcount,mcount-ncount+1,-1
     ii=isort(jj)
     write(message, '(7(i12,a1),f11.2,a2)') &
&     my_distp(1,ii),'|',my_distp(2,ii),'|',my_distp(3,ii),'|',my_distp(4,ii),'|', &
&     my_distp(5,ii),'|',my_distp(6,ii),'|',my_distp(7,ii),'|',weight(jj),' |'
     call wrtout(std_out,message,'COLL')
     if (dtset%paral_kgb<0)then
       call wrtout(ab_out,message,'COLL')
     end if
   end do
 end if
 if (dtset%paral_kgb<0.and.(mcount_eff>MAXCOUNT)) then
   write(message,'(a,i3,a,i3,a)') &
&   ' More than ',MAXCOUNT,' possible choices for nproc; only the first ',&
&   MAXCOUNT,' ones are printed...'
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,message,'COLL')
 end if

 icount=isort(mcount)

!Refinement of the process distribution by mean of a LinAlg routines benchmarking
 if (dtset%autoparal/=0) then
   if (dtset%autoparal/=2) then
     if (dtset%autoparal==1) then
       write(message,'(5a,9(a10,a1))') ch10, &
&       ' Values below have been tested with respect to Linear Algebra performance;',ch10,&
&       ' Weights below are corrected according:',ch10,&
&       'npimage','|','npkpt' ,'|','npspinor'  ,'|','npfft'     ,'|','npband','|',' bandpp ' ,'|',&
&       'nproc'  ,'|','weight','|','new weight','|'
     else
       write(message,'(5a,11(a10,a1))') ch10, &
&       ' Values below have been tested with respect to Linear Algebra performance;',ch10,&
&       ' Weights below are corrected according:',ch10,&
&       'npimage','|','npkpt' ,'|','npspinor'  ,'|','npfft'     ,'|','npband','|',' bandpp ' ,'|',&
&       'nproc'  ,'|','weight','|','new weight','|','best npslk','|','linalggpu' ,'|'
     end if
     call wrtout(std_out,message,'COLL')
     if (dtset%paral_kgb < 0)then
       call wrtout(ab_out,message,'COLL')
     end if
   end if
   acc_k=zero
   ncount=min(MAXBENCH,mcount);if (dtset%autoparal==2) ncount=1
   do jj=mcount,mcount-ncount+1,-1
     ii=isort(jj)
     npf=my_distp(4,ii);npb=my_distp(5,ii);bpp=my_distp(6,ii)
     if ((npb*npf*bpp>1).and.(npf*npb<=mpi_enreg%nproc)) then
       use_linalg_gpu=dtset%use_gpu_cuda
       call compute_kgb_indicator(acc_kgb,bpp,xmpi_world,mband,mpw,npb,npf,np_slk,use_linalg_gpu)
       if (dtset%autoparal/=1) then
         my_distp(9,ii)=np_slk
         if (np_slk>0) my_distp(8,ii)=1
!        * gpu_linalg_limit:
!        No use of GPU: huge value ~2  *vectsize*blocksize**2 tested
!        Use of GPU:    tiny value ~0.5*vectsize*blocksize**2 tested
         my_distp(10,ii)=2*dtset%mpw*(npb*bpp)**2/npf
         if (use_linalg_gpu==1) my_distp(10,ii)=my_distp(10,ii)/4
       end if
       if (abs(acc_k)<=tol12) acc_k=acc_kgb ! Ref value : the first one computed
!      * Weight (corrected by 10% of the computed ratio)
       weight0=weight(jj)*(one + 0.1_dp*acc_k/acc_kgb)
       if (dtset%autoparal==1) then
         write(message, '(7(i10,a1),f9.2,a2,f9.5,a2)') &
&         my_distp(1,ii),'|',my_distp(2,ii),'|',my_distp(3,ii),'|',my_distp(4,ii),'|',&
&         my_distp(5,ii),'|',my_distp(6,ii),'|',my_distp(7,ii),'|',weight(jj),'=>', weight0,' |'
       else if (dtset%autoparal==2) then
         write(message,'(a,5(a,i3))') ch10,' For npband=',npb,', npfft=',npf,' and bandpp=',bpp, &
&         ', compute_kgb_indicator recommends you to set np_slk=',my_distp(9,ii),&
&         ' and use_linalg_gpu=',use_linalg_gpu
       else
         write(message, '(7(i10,a1),f9.2,a2,f9.5,a2,2(i10,a1))') &
&         my_distp(1,ii),'|',my_distp(2,ii),'|',my_distp(3,ii),'|',my_distp(4,ii),'|',&
&         my_distp(5,ii),'|',my_distp(6,ii),'|',my_distp(7,ii),'|',weight(jj),'=>', weight0,' |',&
&         my_distp(9,ii),'|',use_linalg_gpu,'|'
       end if
       call wrtout(std_out,message,'COLL')
       if (dtset%paral_kgb<0)then
         call wrtout(ab_out,message,'COLL')
       end if
!      We store the best value in weight(mcount) and keep icount
       if (weight0 > weight(mcount)) then
         icount=ii;weight(mcount)=weight0
       end if
     end if
   end do
 end if

!Final advice in case of paral_kgb<0
 if (dtset%paral_kgb<0) then
   write(message,'(6a)') ch10,&
&   ' Launch a parallel version of ABINIT with a number of processors among the above list,',ch10,&
&   ' and the associated input variables npkpt, npband, npfft and bandpp. ',ch10,&
&   ' The optimal weight is close to nproc and the higher should be better.'
   call wrtout(std_out,message,'COLL')
   call wrtout(ab_out,message,'COLL')
   iexit=iexit+1
 end if

!Store new process distribution
 if (dtset%paral_kgb>=0) then
!  Work load distribution
   dtset%npimage  = my_distp(1,icount)
   dtset%npkpt    = my_distp(2,icount)
   dtset%npspinor = my_distp(3,icount)
   dtset%npfft    = my_distp(4,icount)
   dtset%npband   = my_distp(5,icount)
   dtset%bandpp   = my_distp(6,icount)
   if (dtset%npband*dtset%npfft*dtset%bandpp>1) dtset%paral_kgb=1
!  LinAlg parameters: we change values only if they are not present in input file
   if (dtset%paral_kgb==1) then
     if (tread(8)==0) dtset%use_slk=my_distp(8,icount)
     if (tread(9)==0) dtset%np_slk=my_distp(9,icount)
     if (tread(10)==0) dtset%gpu_linalg_limit=my_distp(10,icount)
   end if
!  New definition of "world" MPI communicator
   call initmpi_world(mpi_enreg,my_distp(7,icount))
 end if

 ABI_DEALLOCATE(isort)
 ABI_DEALLOCATE(my_distp)
 ABI_DEALLOCATE(weight)

 DBG_EXIT("COLL")

end subroutine finddistrproc
!!***
