!{\src2tex{textfont=tt}}
!!****f* ABINIT/ptildefit
!! NAME
!!  ptildefit
!!
!! FUNCTION
!! Fit tproj to Gaussians
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  basis_size= size of the PAW basis
!!  orbitals= indicates the l quantum number for all orbitals.
!!  rpaw= PAW radius
!!  pawrad <type(pawrad_type)>=paw radial mesh and related data
!!  tproj= projectors
!!  maxterm= maximum number of terms used to fit the projectors.
!!  mparam= maximum number of parameters (Gaussian coefficients and factors) used.
!!
!! OUTPUT
!!  nparam_array= number of parameters found.
!!  param = parameters found  (Gaussian coefficients and factors).
!!
!! NOTES
!! chisq=accuracy_p= sum_x abs(f(x)-y(x))/nx. 
!!    nx is the number of points, f(x) and y(x) are the fitted and original functions.
!!
!! PARENTS
!!      pspatm_abinit
!!
!! CHILDREN
!!      fit_gen,leave_new,pawrad_deducer0,pawrad_destroy,pawrad_init,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine ptildefit(basis_size,mparam,nparam_array,nterm_bounds,orbitals,param,pawrad,&
& rpaw,tproj,comm_mpi)

 use m_profiling
 use defs_basis
 use m_errors
 use m_pawrad, only : pawrad_type, pawrad_init, pawrad_destroy, pawrad_deducer0

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'ptildefit'
 use interfaces_14_hidewrite
 use interfaces_16_hideleave
 use interfaces_64_atompaw
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 integer,intent(in)::basis_size
 integer,intent(in)::orbitals(basis_size)
 integer, optional,intent(in) :: comm_mpi
 real(dp),intent(in)::rpaw
 type(pawrad_type),intent(in) :: pawrad
 real(dp),intent(in)    :: tproj(pawrad%mesh_size,basis_size)
 integer,intent(in)::mparam,nterm_bounds(2)
 !
 integer,intent(out)::nparam_array(basis_size)
 real(dp),intent(out)::param(mparam,basis_size)
 type(pawrad_type)::mesh_tmp

!Local variables ------------------------------
 integer :: ibasis,il
 integer :: msz,nr,option
 real(dp),allocatable::tproj_tmp(:)
 character(len=500) :: message
 character(80)::outfile
 !debug: uncomment
! integer::i,ii,ir,nterm ,unitp
 !real(dp),allocatable::y(:)
 !end debug
!************************************************************************


 option=4  !see fit_gen
 nparam_array(:)=0
 msz=pawrad%mesh_size
!
!augment the mesh size
!this is to make the Gaussians go to zero after paw_radius
!This is done by creating a new pawrad objet: mesh_tmp
 if(pawrad%mesh_type==1) nr=msz*2        !lineargrid
 if(pawrad%mesh_type/=1) nr=int(msz*1.2) !log grid
!
!write(*,*)'Fit done from 0 to',r_fit
 call pawrad_init(mesh_tmp,mesh_size=nr,mesh_type=pawrad%mesh_type,rstep=pawrad%rstep,lstep=pawrad%lstep)

 ABI_ALLOCATE(tproj_tmp,(nr))

 do ibasis=1,basis_size
!  
   write(message,'(a," - Fitting wfn ",i4," to Gaussians")')ch10,ibasis
   call wrtout(std_out,  message,'COLL')
!  
   tproj_tmp(:)=0.d0
!  take out r^il factor:
!  il=psps%indlmn(1,ilmn,itypat)
   il=orbitals(ibasis)

   tproj_tmp(1:msz)=tproj(1:msz,ibasis)/((pawrad%rad(1:msz)+tol8)**(il))

!  take out 1/r factor from eq.(3) of M. Torrent CMS 42, 337 (2008)
!  since: <phi|proj>=1 from atompaw, and phi=phi*r, alors proj=proj/r
   tproj_tmp(2:msz)=tproj_tmp(2:msz)/(pawrad%rad(2:msz)) 

   call pawrad_deducer0(tproj_tmp(1:msz),msz,pawrad)

!  debug
!  write(*,*)'ptildefit: erase me: l2325 debug: write fort.800'
!  i=800+ibasis
!  do ii=1,msz   !nr
!  write(i,*)mesh_tmp%rad(ii),tproj_tmp(ii)
!  end do


!  
!  
!  Obtain the name for the output file
   if(ibasis<10) then
     write(outfile,'("wfn",i1,".fit")')ibasis
   elseif(ibasis<100) then
     write(outfile,'("wfn",i2,".fit")')ibasis
     write(message,'(a,a,a,a)')ch10,&
&     "ptildefit : bug, ib (basis index) is too big!",ch10,&
&     "Action: check your pseudopotentials"
     call wrtout(ab_out,message,'COLL')
     call wrtout(std_out,  message,'COLL')
     call leave_new('COLL')
   end if
!  
   if(present(comm_mpi)) then
     call fit_gen(mparam,nparam_array(ibasis),nterm_bounds,nr,&
&     param(:,ibasis),mesh_tmp,option,outfile,rpaw,tproj_tmp,comm_mpi)
   else
     call fit_gen(mparam,nparam_array(ibasis),nterm_bounds,nr,&
&     param(:,ibasis),mesh_tmp,option,outfile,rpaw,tproj_tmp)
   end if

!  check
!  ABI_ALLOCATE(y,(mesh_tmp%mesh_size))
!  nterm=nparam_array(ibasis)/4
!  call calcgaussc4(nparam_array(ibasis),nterm,mesh_tmp%mesh_size,1,param(:,ibasis),&
!  &  mesh_tmp%rad,y)
!  ! unitp=600+ibasis
!  ! do ir=1,mesh_tmp%mesh_size
!  !  write(unitp,'(2(f16.7,x,f16.7))')mesh_tmp%rad(ir),y(ir)
!  ! end do
!  ABI_DEALLOCATE(y) 
!  
 end do
!
!Deallocate
 call pawrad_destroy(mesh_tmp)
 ABI_DEALLOCATE(tproj_tmp)

end subroutine ptildefit
!!***
